﻿using System;
using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing;
//using System.Linq;
//using System.Text;
using System.Threading.Tasks;

using Newtonsoft.Json;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Web;
using Microsoft.AspNet.SignalR.Client;
using System.Windows.Forms;
using Newtonsoft.Json.Linq;

namespace winform12
{
    //Note before run this app need to:
    //1. run MBL service so that Get token would work
    //2. do get token so that both RestfulAPI endpoint would work, and Alert handler will bind to SignalR methods
    //3. need to run messaging service and HCS/Wireless service so that alert will send to Alter handler

    public partial class Form1 : Form
    {
        //Connection to a SignalR server
        HubConnection _signalRConnection;

        //Proxy object for a hub hosted on the SignalR server
        IHubProxy _hubProxy;


        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //var client = new RestClient("http://localhost:7049/token");
            //client.Timeout = -1;
            //var request = new RestRequest(Method.GET);
            //request.AddHeader("Content-Type", "application/x-www-form-urlencoded");
            //request.AddParameter("username", "admin");
            //request.AddParameter("password", "admin@4k");
            //request.AddParameter("grant_type", "password");
            //IRestResponse response = client.Execute(request);
            //Console.WriteLine(response.Content);


            Task<Token> task = Task.Run<Token>(async () => await GetToken(txtTokenUrl.Text));
            var result = task.Result;
            txtToken.Text = result.AccessToken;
        }

        private static async Task<Token> GetToken(string url)
        {
            var client = new HttpClient();
            string baseAddress = url;       //@"http://localhost:7049/token";

            string grant_type = "password";
            string username = "admin";
            string password = "admin@4k";

            var form = new Dictionary<string, string>
            {
                {"grant_type", grant_type},
                {"username", username},
                {"password", password},
            };

            HttpResponseMessage tokenResponse = await client.PostAsync(baseAddress, new FormUrlEncodedContent(form));
            var jsonContent = await tokenResponse.Content.ReadAsStringAsync();
            Token tok = JsonConvert.DeserializeObject<Token>(jsonContent);
            return tok;
        }


        #region "Restful API"
        private const string RESPONSE_NOT_SUCCESS = "Response Not Success";
        private void button3_Click(object sender, EventArgs e)
        {
            var token = txtToken.Text;
            var url = txtUrl.Text;
            var endpoint = "";

            //HttpGet
            //Badges/BadgeInfo
            var badge = "7280";
            var facility = "0";
            endpoint = $@"Badges/BadgeInfo?badge={badge}&facility={facility}";

            //Task<string> task = Task.Run<string>(async () => await GetBadgeInfo(token, url, endpoint));
            var str = GetBadgeInfo(token, url, endpoint).ConfigureAwait(true).GetAwaiter().GetResult();
            txtHttpGetResponse.Text = str;
        }

        public static async Task<string> GetBadgeInfo(string authorizeToken, string url, string endpoint)
        {
            string sret = "";
            using (var client = new HttpClient())
            {
                string authorization = authorizeToken;
                // Setting Authorization.  
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", authorization);

                // Setting Base address.  
                client.BaseAddress = new Uri(url);

                // Setting content type.  
                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

                // Initialization.  
                var response = new HttpResponseMessage();

                // HTTP GET  
                response = await client.GetAsync(endpoint).ConfigureAwait(false);

                if (!response.IsSuccessStatusCode)
                {
                    sret = RESPONSE_NOT_SUCCESS;
                }
                else
                {
                    //var result = await response.Content.ReadAsAsync<string>();
                    var result = await response.Content.ReadAsStringAsync();
                    sret = result;
                }
            }
            return sret;
        }

        private void button4_Click(object sender, EventArgs e)
        {
            var token = txtToken.Text;
            var url = txtUrl.Text;
            var endpoint = "";

            //HttPost
            //Reports/Events
            //sample url parameters
            string startDate = "01-15-2020";
            string endDate = "12-30-2020";
            endpoint = $@"Reports/Events?startDate={startDate}&endDate={endDate}";

            //sample body parameters
            var data = new Dictionary<string, string>
            {
                { "Badge", "" },
                { "CompanyName", "15265-Company" }
            };
            var body = new FormUrlEncodedContent(data);
            var str = GetEvents(token, url, endpoint, body).ConfigureAwait(true).GetAwaiter().GetResult();
            txtHttpPostResponse.Text = str;
        }

        public static async Task<string> GetEvents(string authorizeToken, string url, string endpoint, FormUrlEncodedContent body)
        {
            string sret = "";
            using (var client = new HttpClient())
            {
                string authorization = authorizeToken;
                // Setting Authorization.  
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", authorization);

                // Setting Base address.  
                client.BaseAddress = new Uri(url);

                // Setting content type.  
                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

                // Initialization.  
                var response = new HttpResponseMessage();

                // HTTP Post  
                response = await client.PostAsync(endpoint, body).ConfigureAwait(false);
                if (!response.IsSuccessStatusCode)
                {
                    sret = RESPONSE_NOT_SUCCESS;
                }
                else
                {
                    var result = await response.Content.ReadAsStringAsync();
                    sret = result;
                }
            }
            return sret;
        }
        #endregion

        #region "test SignalR client"
        private async void button2_Click(object sender, EventArgs e)
        {
            //connect to the SignalR hub server
            await connectAsync();
        }
        private async Task connectAsync()
        {
            //create a connection for the SignalR server
            _signalRConnection = new HubConnection(txtSinalRUrl.Text);
            _signalRConnection.StateChanged += HubConnection_StateChanged;

            //get a proxy object that will be used to interact with the specific hub on the server
            //there may be many hubs hosted on the server, so provide the type name for the hub
            _hubProxy = _signalRConnection.CreateHubProxy("alertHubService");

            //register to the "broadcastmessage" callback method of the hub
            _hubProxy.On<string>("broadcastmessage", (message) => displyMessage(1, $"broadcastmessage:{message}"));

            //register to the "alertReceived" callback method of the hub, this is event alert send from hub server
            _hubProxy.On<string>("alertReceived", (message) => displyMessage(2, $"alertReceived:{message}"));

            try
            {
                //connect to the hub server
                await _signalRConnection.Start();
                //hook Ca4k message event
                await _hubProxy.Invoke("Connect");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error:{ex.Message}");
            }
        }
        private void HubConnection_StateChanged(StateChange obj)
        {
            if (obj.NewState == Microsoft.AspNet.SignalR.Client.ConnectionState.Connected)
                displyMessage(1, "Connected");
            else if (obj.NewState == Microsoft.AspNet.SignalR.Client.ConnectionState.Disconnected)
                displyMessage(2, "Disconnected");
        }
        private void displyMessage(int flag, string message)
        {
            if (flag == 1)
            {
                if (this.InvokeRequired)
                    this.BeginInvoke(new Action(() => txtLog.AppendText($"{message}{Environment.NewLine}")));
                else
                    txtLog.AppendText($"{message}{Environment.NewLine}");
            }
            else
            {
                if (this.InvokeRequired)
                    this.BeginInvoke(new Action(() => txtAlerts.AppendText($"{message}{Environment.NewLine}{Environment.NewLine}")));
                else
                    txtAlerts.AppendText($"{message}{Environment.NewLine}{Environment.NewLine}");
            }
        }
        #endregion

        private void button5_Click(object sender, EventArgs e)
        {
            var token = txtToken.Text;
            var url = txtUrl.Text;
            var endpoint = "";

            //HttpGet
            //Badges/BadgeInfo
            //var searchField = "CompanyName";
            //var searchValue = "AEL";
            endpoint = $@"Badges/BadgeInfoByField?searchField={txtSearchField.Text}&searchValue={txtSearchValue.Text}";

            //Task<string> task = Task.Run<string>(async () => await GetBadgeInfo(token, url, endpoint));
            var str = GetBadgeInfo(token, url, endpoint).ConfigureAwait(true).GetAwaiter().GetResult();
            textBadgeInfoByField.Text = str;
        }
        public static async Task<string> GetBadgeInfo(string authorizeToken, string url, string endpoint, FormUrlEncodedContent body)
        {
            string sret = "";
            using (var client = new HttpClient())
            {
                string authorization = authorizeToken;
                // Setting Authorization.  
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", authorization);

                // Setting Base address.  
                client.BaseAddress = new Uri(url);

                // Setting content type.  
                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

                // Initialization.  
                var response = new HttpResponseMessage();

                // HTTP GET  
                response = await client.GetAsync(endpoint).ConfigureAwait(false);

                if (!response.IsSuccessStatusCode)
                {
                    sret = RESPONSE_NOT_SUCCESS;
                }
                else
                {
                    //var result = await response.Content.ReadAsAsync<string>();
                    var result = await response.Content.ReadAsStringAsync();
                    sret = result;
                }
            }
            return sret;
        }

    }
    internal class Token
    {
        [JsonProperty("access_token")]
        public string AccessToken { get; set; }

        [JsonProperty("token_type")]
        public string TokenType { get; set; }

        [JsonProperty("expires_in")]
        public int ExpiresIn { get; set; }

        [JsonProperty("refresh_token")]
        public string RefreshToken { get; set; }
    }
}
